package net.knarcraft.ffmpegconverter.streams;

import net.knarcraft.ffmpegconverter.utility.FileUtil;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

/**
 * An object representation of a subtitle stream in a media file
 */
public class SubtitleStream extends AbstractStream implements StreamObject {

    final private String title;
    final private String file;
    final private boolean isFullSubtitle;
    final private boolean isImageSubtitle;
    final private boolean isInternalStream;

    /**
     * Instantiates a new subtitle stream
     *
     * @param codecName        <p>The name of the codec for the subtitle stream.</p>
     * @param absoluteIndex    <p>The index of the subtitle stream.</p>
     * @param relativeIndex    <p>The index of the subtitle stream relative to other subtitle streams.</p>
     * @param language         <p>The language of the subtitle stream.</p>
     * @param title            <p>The title of the subtitle stream.</p>
     * @param file             <p>The file containing the subtitle.</p>
     * @param isInternalStream <p>Whether this subtitle stream is in the video file itself</p>
     */
    public SubtitleStream(@Nullable String codecName, int absoluteIndex, int relativeIndex, @Nullable String language,
                          @Nullable String title, @NotNull String file, boolean isInternalStream) {
        this.codecName = codecName;
        this.absoluteIndex = absoluteIndex;
        this.language = language;
        this.title = title;
        this.isFullSubtitle = isFullSubtitle();
        this.relativeIndex = relativeIndex;
        this.isImageSubtitle = isImageSubtitle();
        this.isInternalStream = isInternalStream;
        this.file = file;

        if (this.language == null || this.language.isEmpty()) {
            String possibleLanguage = FileUtil.getLanguage(file);
            if (possibleLanguage != null) {
                this.language = possibleLanguage;
            }
        }
    }

    /**
     * Gets the title of the subtitle stream
     *
     * @return <p>The title of the subtitle stream.</p>
     */
    public String getTitle() {
        return this.title;
    }

    /**
     * Gets the file name of the file containing this subtitle
     *
     * @return <p>The file name containing the subtitle stream.</p>
     */
    public String getFile() {
        return this.file;
    }

    /**
     * Gets whether this subtitle stream is an internal stream, not an external one
     *
     * @return <p>True if this stream is an internal stream</p>
     */
    public boolean isInternalSubtitle() {
        return this.isInternalStream;
    }

    /**
     * Gets whether the subtitle is an image subtitle
     *
     * @return <p>Whether the subtitles is an image subtitle.</p>
     */
    public boolean getIsImageSubtitle() {
        return this.isImageSubtitle;
    }

    /**
     * Gets whether the subtitle is a full subtitle
     *
     * @return <p>Whether the subtitle is a full subtitle.</p>
     */
    public boolean getIsFullSubtitle() {
        return this.isFullSubtitle;
    }

    /**
     * Checks whether a subtitle is image based (as opposed to text based)
     *
     * @return <p>True if the subtitle is image based.</p>
     */
    private boolean isImageSubtitle() {
        return codecName != null && (getCodecName().equals("hdmv_pgs_subtitle")
                || getCodecName().equals("dvd_subtitle"));
    }

    /**
     * Checks whether the subtitle translates everything (as opposed to just songs and signs)
     *
     * @return <p>True if the subtitle translates everything.</p>
     */
    private boolean isFullSubtitle() {
        if (getTitle() == null) {
            return false;
        }
        String titleLowercase = getTitle().toLowerCase();
        return !titleLowercase.matches(".*si(ng|gn)s?[ &/a-z]+songs?.*") &&
                !titleLowercase.matches(".*songs?[ &/a-z]+si(gn|ng)s?.*") &&
                !titleLowercase.matches(".*forced.*") &&
                !titleLowercase.matches(".*s&s.*") &&
                !titleLowercase.matches("signs?") &&
                !titleLowercase.matches("songs?") &&
                !titleLowercase.contains("signs only");
    }

}